﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;

namespace howto_save_as
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The document's file name.
        private string _FileName = "";
        private string FileName
        {
            get { return _FileName; }
            set
            {
                if (value == _FileName) return;
                _FileName = value;
                SetFormCaption();
            }
        }

        // True if the document was changed since opening/creation.
        private bool _DocumentChanged = false;
        private bool DocumentChanged
        {
            get { return _DocumentChanged; }
            set
            {
                if (value == _DocumentChanged) return;
                _DocumentChanged = value;
                SetFormCaption();
            }
        }

        // Set the form's caption to indicate the file
        // name and whether there are unsaved changes.
        private void SetFormCaption()
        {
            string caption = "howto_save_as ";

            // If there are changes, add an asterisk.
            if (DocumentChanged) caption += "* ";
            else caption += " ";

            // Add the file name without its path.
            if (FileName == "") caption += "[ ]";
            else caption += "[" + Path.GetFileName(FileName) + "]";

            // Display the result.
            this.Text = caption;
        }

        // Return true if it is safe to discard the current document.
        private bool IsDocumentSafe()
        {
            // If there are no current changes to the document, it's safe.
            if (!DocumentChanged) return true;

            // See if the user wants to save the changes.
            switch (MessageBox.Show(
                "There are unsaved changes. Do you want to save the document?",
                "Save Changes?", MessageBoxButtons.YesNoCancel))
            {
                case DialogResult.Yes:
                    // Save the changes.
                    SaveDocument();

                    // If the document still has unsaved changes,
                    // then we didn't save so the document is not safe.
                    return (!DocumentChanged);

                case DialogResult.No:
                    // It's safe to lose the current changes.
                    return true;

                default:
                    // Cancel. It's not safe to lose the changes.
                    return false;
            }
        }

        // Attempt to save the document.
        // If we don't have a file name, treat this as Save As.
        private void SaveDocument()
        {
            // See if we have a file name.
            if (FileName == "")
            {
                // We have no file name. Treat as Save As.
                SaveDocumentAs();
            }
            else
            {
                // Save with the current name.
                SaveDocumentAs(FileName);
            }
        }

        // Attempt to save the document with a new file name.
        private void SaveDocumentAs()
        {
            // Let the user pick the file in which to save.
            if (sfdDocument.ShowDialog() == DialogResult.OK)
            {
                // Save using the selected file name.
                SaveDocumentAs(sfdDocument.FileName);
            }
        }

        // Save the document with the indicated name.
        private void SaveDocumentAs(string file_name)
        {
            // Save with the indicated name.
            try
            {
                // Save the file.
                if (Path.GetExtension(file_name).ToLower() == ".txt")
                {
                    // Save as a text file.
                    File.WriteAllText(file_name, rchDocument.Text);
                }
                else
                {
                    // Save as an RTF file.
                    rchDocument.SaveFile(file_name);
                }

                // Update the document's file name.
                FileName = file_name;

                // There are no unsaved changed now.
                DocumentChanged = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        // Create a new document.
        private void mnuFileNew_Click(object sender, EventArgs e)
        {
            // Make sure it's safe to discard the current document.
            if (!IsDocumentSafe()) return;

            // Clear the document.
            rchDocument.Clear();

            // There are no changes.
            DocumentChanged = false;

            // We have no file name yet.
            FileName = "";
        }

        // Open an existing file.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            // Make sure it's safe to discard the current document.
            if (!IsDocumentSafe()) return;

            // Let the user pick the file.
            if (ofdDocument.ShowDialog() == DialogResult.OK)
            {
                // Open the file.
                OpenFile(ofdDocument.FileName);
            }
        }

        // Open the indicated file.
        private void OpenFile(string file_name)
        {
            try
            {
                // Load the file.
                if (Path.GetExtension(file_name).ToLower() == ".txt")
                {
                    // Read as a text file.
                    rchDocument.Text = File.ReadAllText(file_name);
                }
                else
                {
                    // Read as an RTF file.
                    rchDocument.LoadFile(file_name);
                }

                // There are no changes.
                DocumentChanged = false;

                // Save the file's name.
                FileName = file_name;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        // Save using the current document name.
        private void mnuFileSave_Click(object sender, EventArgs e)
        {
            SaveDocument();
        }

        // Save the document with a new name.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveDocumentAs();
        }

        // Exit. Just try to close.
        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Flag the document as changed.
        private void rchDocument_TextChanged(object sender, EventArgs e)
        {
            DocumentChanged = true;
        }

        // Initially we have a new unchanged document.
        private void Form1_Load(object sender, EventArgs e)
        {
            mnuFileNew_Click(null, null);
        }

        // If it's not safe to close, cancel the close.
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !IsDocumentSafe();
        }
    }
}
